# Generated by roxygen2: do not edit by hand

S3method(extract_at,SFCB)
S3method(plot,SFCB)
S3method(print,SFCB)
S3method(print,SISIRres)
S3method(print,ridgeRes)
S3method(print,sparseRes)
S3method(project,sparseRes)
S3method(quality,SFCB)
S3method(summary,SFCB)
S3method(summary,SISIRres)
S3method(summary,ridgeRes)
S3method(summary,sparseRes)
export(SISIR)
export(extract_at)
export(project)
export(quality)
export(ridgeSIR)
export(sfcb)
export(sparseSIR)
export(tune.ridgeSIR)
import(dendextend)
import(doParallel)
import(foreach)
import(ggplot2)
import(glmnet)
import(magrittr)
importFrom(CORElearn,attrEval)
importFrom(Matrix,forceSymmetric)
importFrom(RColorBrewer,brewer.pal)
importFrom(RSpectra,eigs)
importFrom(adjclust,adjClust)
importFrom(adjclust,cutree_chac)
importFrom(aricode,AMI)
importFrom(aricode,ARI)
importFrom(aricode,NID)
importFrom(aricode,NMI)
importFrom(aricode,NVI)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(expm,sqrtm)
importFrom(graphics,abline)
importFrom(mixOmics,pls)
importFrom(parallel,detectCores)
importFrom(purrr,map_dfc)
importFrom(ranger,ranger)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(stats,as.dendrogram)
importFrom(stats,as.hclust)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(tidyselect,starts_with)
importFrom(utils,head)
importFrom(utils,packageVersion)
