% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparseSIR.R
\name{sparseRes}
\alias{sparseRes}
\alias{summary.sparseRes}
\alias{print.sparseRes}
\alias{sparseRes-class}
\title{Print sparseRes object}
\usage{
\method{summary}{sparseRes}(object, ...)

\method{print}{sparseRes}(x, ...)
}
\arguments{
\item{object}{a \code{sparseRes} object}

\item{...}{not used}

\item{x}{a \code{sparseRes} object}
}
\description{
Print a summary of the result of \code{\link{sparseSIR}} (
\code{sparseRes} object)
}
\seealso{
\code{\link{sparseSIR}}
}
\author{
{Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inra.fr}}
}
