% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparseSIR.R
\name{SISIRres}
\alias{SISIRres}
\alias{summary.SISIRres}
\alias{print.SISIRres}
\title{Print SISIRres object}
\usage{
\method{summary}{SISIRres}(object, ...)

\method{print}{SISIRres}(x, ...)
}
\arguments{
\item{object}{a \code{SISIRres} object}

\item{...}{not used}

\item{x}{a \code{SISIRres} object}
}
\description{
Print a summary of the result of \code{\link{SISIRres}} (
\code{SISIRres} object)
}
\seealso{
\code{\link{SISIR}}
}
\author{
{Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
}
